#include "ndewidgetmaterialrippleoverlay.h"
#include <QPainter>
#include <QEvent>
#include "ndewidgetmaterialripple.h"


namespace NdeWidget
{
QtMaterialOverlayWidget::QtMaterialOverlayWidget(QWidget *parent)
    : QWidget(parent)
{
    QRect rect = parent->rect();
    if (parent)
    {
        parent->installEventFilter(this);
    }
}

QtMaterialOverlayWidget::~QtMaterialOverlayWidget()
{
}

/*!
 *  \reimp
 */
bool QtMaterialOverlayWidget::event(QEvent *event)
{
    if (!parent())
    {
        return QWidget::event(event);
    }
    switch (event->type())
    {
        case QEvent::ParentChange:
        {
            parent()->installEventFilter(this);
            setGeometry(overlayGeometry());
            break;
        }
        case QEvent::ParentAboutToChange:
        {
            parent()->removeEventFilter(this);
            break;
        }
        default:
            break;
    }
    return QWidget::event(event);
}

/*!
 *  \reimp
 */
bool QtMaterialOverlayWidget::eventFilter(QObject *obj, QEvent *event)
{
    switch (event->type())
    {
    case QEvent::Move:
    case QEvent::Resize:
        setGeometry(overlayGeometry());
        break;
    default:
        break;
    }
    return QWidget::eventFilter(obj, event);
}

QRect QtMaterialOverlayWidget::overlayGeometry() const
{
    QWidget *widget = parentWidget();
    if (!widget) {
        return QRect();
    }
    return widget->rect();
}
QtMaterialRippleOverlay::QtMaterialRippleOverlay(QWidget *parent)
    : QtMaterialOverlayWidget(parent)
{
    setAttribute(Qt::WA_TransparentForMouseEvents);
    setAttribute(Qt::WA_NoSystemBackground);
}

QtMaterialRippleOverlay::~QtMaterialRippleOverlay()
{
}

void QtMaterialRippleOverlay::removeRipple(MaterialRipple *ripple)
{
    if (m_ripples.removeOne(ripple)) {
        delete ripple;
        update();
    }
}


QtMaterialCircleRippleOverlay::QtMaterialCircleRippleOverlay(QWidget * parent)
    :QtMaterialRippleOverlay(parent)
{

}



void QtMaterialCircleRippleOverlay::addRipple(MaterialRipple *ripple)
{
    ripple->setOverlay(this);
    m_ripples.push_back(ripple);
    ripple->start();

    connect(this, SIGNAL(destroyed(QObject*)), ripple, SLOT(stop()));
    connect(this, SIGNAL(destroyed(QObject*)), ripple, SLOT(deleteLater()));
}

/*!
 *  \reimp
 */
void QtMaterialCircleRippleOverlay::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)

    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setPen(Qt::NoPen);

    QList<MaterialRipple *>::const_iterator i;
    for (i = m_ripples.begin(); i != m_ripples.end(); ++i) {
        paintRipple(&painter, *i);
    }
}

void QtMaterialCircleRippleOverlay::paintRipple(QPainter *painter, MaterialRipple *ripple)
{
    const qreal radius = ripple->radius();
    const QPointF center = ripple->center();
    painter->setOpacity(ripple->opacity());
    painter->setBrush(ripple->brush());
    painter->drawEllipse(center, radius, radius);
}
}
