#include "mainwindow.h"
#include "./ui_mainwindow.h"
#include "designer/ndewidgetmaterialtoggle.h"
#include "designer/ndewidgetimagelist.h"
#include "designer/ndewidgetmaterialcircularprogress.h"
#include <QVBoxLayout>
#include <QPushButton>
#include <QDir>
#include <QDebug>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    mToggle1 = new NdeWidget::QMaterialToggle(this);
    ui->verticalLayout_2->addWidget(mToggle1);

    mToggle2 = new NdeWidget::QMaterialToggle(this);
    mToggle2->setOrientation(Qt::Vertical);
    ui->verticalLayout->addWidget(mToggle2);

    connect(ui->checkBox_toggleBtn, &QCheckBox::stateChanged, this , &MainWindow::updateWidget);

    circularProgress = new NdeWidget::QMaterialCircularProgress(this);
    circularProgress->setOrientation(Qt::Vertical);
    ui->verticalLayout_4->addWidget(circularProgress);

    QVBoxLayout * listLayout = new QVBoxLayout(ui->frame_list);
    NdeWidget::ImageList * list= new NdeWidget::ImageList(this);
    listLayout->addWidget(list);

    QDir saveDir = QDir(":screensaver-shots/");
    QStringList namelist = saveDir.entryList(QDir::Files | QDir::NoDot | QDir::NoDotAndDotDot | QDir::Readable);
    foreach(QString name , namelist)
    {
        QString path = saveDir.absoluteFilePath(name);
        QString itemName = name.split(".").takeFirst();
        list->addImage(path,itemName);
    }

    ui->spinBox_2->setRange(10, 200);
    ui->horizontalSlider->setRange(0, 100);
    ui->horizontalSlider_2->setRange(1000, 5000);

    switch (circularProgress->getProgressType())
    {
    case NdeWidget::Material::DeterminateProgress:
        ui->checkBox->setChecked(0);
        break;
    case NdeWidget::Material::IndeterminateProgress:
        ui->checkBox->setChecked(1);
        break;
    default:
        break;
    }

    ui->checkBox_2->setChecked(!circularProgress->isEnabled());
    ui->horizontalSlider->setValue(circularProgress->value());
    ui->horizontalSlider_2->setValue(circularProgress->getDuration());
    ui->spinBox->setValue(circularProgress->getLineWidth());
    ui->spinBox_2->setValue(circularProgress->getSize());

    connect(ui->checkBox_2, SIGNAL(toggled(bool)), this, SLOT(updateWidget()));
    connect(ui->checkBox, SIGNAL(toggled(bool)), this, SLOT(updateWidget()));
    connect(ui->horizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(updateWidget()));
    connect(ui->spinBox, SIGNAL(valueChanged(int)), this, SLOT(updateWidget()));
    connect(ui->horizontalSlider_2, SIGNAL(valueChanged(int)), this, SLOT(updateWidget()));
    connect(ui->spinBox_2, SIGNAL(valueChanged(int)), this, SLOT(updateWidget()));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::updateWidget()
{
    switch (ui->checkBox->isChecked())
    {
    case 0:
        circularProgress->setProgressType(NdeWidget::Material::DeterminateProgress);
        break;
    case 1:

        circularProgress->setProgressType(NdeWidget::Material::IndeterminateProgress);
        break;
    default:
        break;
    }
    if(ui->checkBox_toggleBtn->isChecked())
    {
        mToggle1->setChecked(false);
        mToggle2->setChecked(true);
    }
    else
    {
        mToggle1->setChecked(true);
        mToggle2->setChecked(false);
    }

    circularProgress->setDisabled(ui->checkBox_2->isChecked());
    circularProgress->setValue(ui->horizontalSlider->value());
    circularProgress->setLineWidth(ui->spinBox->value());
    circularProgress->setSize(ui->spinBox_2->value());
    circularProgress->setDuration(ui->horizontalSlider_2->value());
}
