#include "ndewidgetimagelist.h"
#include "ndewidgetimagelistitemwidget.h"
#include <QListWidgetItem>
#include <QResizeEvent>
#include <QDebug>
#include <QScrollBar>

namespace NdeWidget {

ImageList::ImageList(QWidget *parent) : QListWidget(parent)
{
    this->setMovement(Static);
    this->setSelectionMode(SingleSelection);
    this->setResizeMode(Adjust);
    this->setSizeAdjustPolicy(AdjustToContents);
    this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setViewMode(IconMode);
    this->setLayoutMode(Batched);
    this->setDragEnabled(false);
    this->setFlow(LeftToRight);
    this->verticalScrollBar()->setSingleStep(10);
    this->setSpacing(10);

    m_itemSize = QSize(250,200);
    m_imageSize = QSize(200,150);

    this->setGridSize(m_itemSize);

    connect(this, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*))
            ,this, SLOT(slotCurrentChanged(QListWidgetItem*,QListWidgetItem*)));
}

void ImageList::addImage(const QString &path, QString name)
{
    if(path.isEmpty()) return;

    QListWidgetItem * item = new QListWidgetItem(this);

    ImageListItemWidget * itemWgt = new ImageListItemWidget(this);
    itemWgt->setPath(path);
    itemWgt->setText(name);
    itemWgt->setImageSize(m_imageSize);

    item->setSizeHint(m_itemSize);

    this->setItemWidget(item, itemWgt);
}

QSize ImageList::itemSize() const
{
    return m_itemSize;
}

void ImageList::setItemSize(const QSize size)
{
    if(m_itemSize == size) return;
    m_itemSize = size;

    repaint();

    emit itemSizeChanged(size);
}

QSize ImageList::imageSize() const
{
    return m_imageSize;
}

void ImageList::setImageSize(const QSize size)
{
    if(m_imageSize == size) return;
    m_imageSize = size;

    for (int i = 0 ; i < this->count(); i++)
    {
        QListWidgetItem* item = this->item(i);
        ImageListItemWidget* itemWgt = (ImageListItemWidget*)this->itemWidget(item);
        itemWgt->setImageSize(size);
    }

    emit imageSizeChanged(size);
}

bool ImageList::textVisible() const
{
    return m_textVisible;
}

void ImageList::setTextVisible(const bool visible)
{
    if(m_textVisible == visible) return;
    m_textVisible = visible;

    for (int i = 0 ; i < this->count(); i++)
    {
        QListWidgetItem* item = this->item(i);
        ImageListItemWidget* itemWgt = (ImageListItemWidget*)this->itemWidget(item);
        itemWgt->setTextVisible(visible);
    }
    emit textVisibleChanged(visible);
}

void ImageList::slotCurrentChanged(QListWidgetItem *current, QListWidgetItem *pre)
{
    for (int i = 0 ; i < this->count(); i++)
    {
        QListWidgetItem* item = this->item(i);
        ImageListItemWidget* itemWgt = (ImageListItemWidget*)this->itemWidget(item);

        if(current == item)
        {
            itemWgt->setSelected(true);
        }
        else
        {
            itemWgt->setSelected(false);
        }
    }
}

void ImageList::resizeEvent(QResizeEvent *e)
{
    QSize size = this->size();
    int verticalScrollBarWidth = this->verticalScrollBar()->isVisible() ? this->verticalScrollBar()->width():0;
    int space = this->spacing();
    QSize newGridSize = m_itemSize;

    int col = (size.width()-verticalScrollBarWidth-space)/(m_itemSize.width()+space);
    int leftSpace = size.width()-verticalScrollBarWidth  -space - (m_itemSize.width()+space)*col;
    if(col > 0 )
    {
        newGridSize = QSize(m_itemSize.width() + leftSpace/col + space, m_itemSize.height());
        this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    }
    else
    {
        this->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    }

    this->setGridSize(newGridSize);

//    for (int i = 0 ; i < this->count(); i++)
//    {
//        QListWidgetItem* item = this->item(i);
//        //这里需要-2，不然无法放下col列图标
//        item->setSizeHint(QSize(newItemSize.width()-5,newItemSize.height()));
//    }

    QListWidget::resizeEvent(e);
}
}
