#include "ndewidgetimagelistitemwidget.h"
#include <QPainter>
#include <QFile>
#include <QGraphicsDropShadowEffect>

namespace NdeWidget {

#define TEXT_HEIGHT 30
#define SPACE_BETWEEN 4
#define PEN_FRAME 4

ImageListItemWidget::ImageListItemWidget(QWidget *parent) : QWidget(parent)
{
    m_selected = false;
    m_textVisible = true;
    m_imageSize = QSize(200,150);

    QGraphicsDropShadowEffect * effect = new QGraphicsDropShadowEffect(this);
    effect->setColor("#000000");
    effect->setOffset(0);
    effect->setBlurRadius(2);
    this->setGraphicsEffect(effect);
}

QString ImageListItemWidget::path() const
{
    return m_path;
}

void ImageListItemWidget::setPath(const QString path)
{
    if(path.isEmpty() || m_path == path) return;

    QFile f(path);
    if(f.exists())
    {
        m_path = path;

        m_pixmap = QPixmap(path);

        emit pathChanged(path);

        repaint();
    }
}

QPixmap ImageListItemWidget::pixmap() const
{
    return m_pixmap;
}

void ImageListItemWidget::setPixmap(const QPixmap pixmap)
{
    m_pixmap = pixmap;
    emit pixmapChanged(pixmap);
    repaint();
}

QString ImageListItemWidget::text() const
{
    return m_text;
}

void ImageListItemWidget::setText(const QString text)
{
    if(m_text == text) return;
    m_text = text;
    emit textChanged(text);
    repaint();
}

bool ImageListItemWidget::textVisible() const
{
    return m_textVisible;
}

void ImageListItemWidget::setTextVisible(const bool visible)
{
    if(m_textVisible == visible) return;
    m_textVisible = visible;
    emit textVisibleChanged(visible);
    repaint();
}

QSize ImageListItemWidget::imageSize() const
{
    return m_imageSize;
}

void ImageListItemWidget::setImageSize(const QSize size)
{
    if(m_imageSize == size) return;
    m_imageSize = size;
    emit imageSizeChanged(size);
    repaint();
}

bool ImageListItemWidget::selected() const
{
    return m_selected;
}

void ImageListItemWidget::setSelected(const bool sel)
{
    if(m_selected == sel ) return;
    m_selected = sel;
    emit selectedChanged(sel);
    repaint();
}

void ImageListItemWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);//消锯齿
    painter.beginNativePainting();

    QRect rc = this->rect();

    //画选择框
    if(m_selected)
    {
        painter.setPen(QPen(QColor("#2c9dfc"),PEN_FRAME));
        painter.setBrush(Qt::NoBrush);
        painter.drawRect(rc);
    }

    //画图形
    if(!m_pixmap.isNull())
    {
        QRect pixmapRc = getPixmapRect();
        painter.setPen(Qt::NoPen);
        painter.setBrush(Qt::NoBrush);
        painter.drawPixmap(pixmapRc, m_pixmap);

//        painter.setPen(QPen(Qt::black, 2));
//        painter.setBrush(Qt::NoBrush);
//        painter.drawRect(pixmapRc);
    }

    //画文字
    if(m_textVisible)
    {
        QRect textRc = getTextRect();
        painter.setPen(Qt::black);
        painter.setBrush(Qt::NoBrush);
        QTextOption option(Qt::AlignCenter);
        option.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
        painter.drawText(textRc, m_text,option);
    }

    painter.endNativePainting();
//    QWidget::paintEvent(event);
}

QRect ImageListItemWidget::getPixmapRect()
{
    QRect rc = this->rect();
    QRect imageRc = rc.adjusted(4,4,-4,m_textVisible ? -TEXT_HEIGHT:0);

    if(imageRc.height() > m_imageSize.height() )
    {
        int offset = (imageRc.height() - m_imageSize.height())/2;
        imageRc.adjust(0,offset, 0, -offset);
    }
    if(imageRc.width() > m_imageSize.width())
    {
        int offset = (imageRc.width()-m_imageSize.width())/2;
        imageRc.adjust(offset, 0, -offset, 0);
    }
    return imageRc;
}

QRect ImageListItemWidget::getTextRect()
{
    QRect rc = this->rect();
    QRect textRc = rc.adjusted(4,rc.height() - TEXT_HEIGHT,-4,-4);

    return textRc;
}

}
