#include "ndewidgetmaterialtoggle.h"
#include <QPainter>
#include <QMouseEvent>
#include "animations/ndewidgetmaterialrippleoverlay.h"
#include "animations/ndewidgetmaterialripple.h"
#include "animations/ndewidgetanimationmanager.h"
namespace NdeWidget {

QMaterialToggle::QMaterialToggle(QWidget *parent)
    : QWidget(parent),
      m_nHeight(16),
      m_bChecked(false),
      m_radius(6.0),
      m_nMargin(3),
      m_BackgroundColor(QColor("#011222")),
      m_thumbColor(QColor("#ffffff")),
      m_checkedColor(QColor("#2c9dfc")),
      m_orientation(Qt::Horizontal)
{
    //添加动画显示窗口到动画管理器
    ndewidgetanimationmanager().getCirclemanager().registerWidget(this,new QtMaterialCircleRippleOverlay(parent));
    // 鼠标滑过光标形状 - 手型
    setCursor(Qt::PointingHandCursor);
}
QColor overlayColors(const QColor &base, const QColor &paint,
                                       QPainter::CompositionMode comp = QPainter::CompositionMode_SourceOver)
{
    QImage img(1, 1, QImage::Format_ARGB32_Premultiplied);
    QPainter p(&img);
    QColor start = base;
    start.setAlpha(255); // opaque
    p.fillRect(0, 0, 1, 1, start);
    p.setCompositionMode(comp);
    p.fillRect(0, 0, 1, 1, paint);
    p.end();
    return img.pixel(0, 0);
}

// 绘制开关
void QMaterialToggle::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);

    QPainter painter(this);
    painter.setPen(Qt::NoPen);
    painter.setRenderHint(QPainter::Antialiasing);

    QPainterPath path;
    QColor background;
    QColor thumbColor;
    qreal dOpacity;
    qreal thumbOpacity;

    if (isEnabled()) { // 可用状态
        if (m_bChecked) { // 打开状态
            background = m_checkedColor;
            background.setAlphaF(0.6);
            thumbColor = m_checkedColor;

            //dOpacity = 0.6;
            //thumbOpacity = 1;
        } else { //关闭状态
            background = m_BackgroundColor;
            background.setAlphaF(0.25);
            thumbColor = m_thumbColor;
            //dOpacity = 0.25;
            //thumbOpacity =1;
        }
    } else {  // 不可用状态

        if (m_bChecked) { // 打开状态
            background = m_checkedColor;
            background.setAlphaF(0.4);
            thumbColor = m_checkedColor;
            thumbColor.setAlphaF(0.2);
            //dOpacity = 0.4;
            //thumbOpacity = 0.2;
        } else { //关闭状态
            background = m_BackgroundColor;
            background.setAlphaF(0.1);
            QColor color = m_BackgroundColor;
            color.setAlphaF(0.05);
            thumbColor = overlayColors(m_thumbColor,color);
            //dOpacity = 0.1;
            //thumbOpacity = 1;
        }
    }
    if(m_orientation == Qt::Horizontal) {
        // 绘制大椭圆
        painter.setBrush(background);
        //painter.setOpacity(dOpacity);
        path.addRoundedRect(QRectF(m_nMargin, m_nMargin, width() - 2 * m_nMargin , height() - 2 * m_nMargin), m_radius, m_radius);
        painter.drawPath(path.simplified());

        // 绘制小椭圆
        painter.setBrush(thumbColor);
        //painter.setOpacity(thumbOpacity);
        painter.drawEllipse(QPoint(m_nX,qreal(height())/2 + 0.5),height()/2,height()/2);
    } else {
        // 绘制大椭圆
        painter.setBrush(background);
        //painter.setOpacity(dOpacity);
        path.addRoundedRect(QRectF(m_nMargin, m_nMargin, width() - 2 * m_nMargin , height() - 2 * m_nMargin), m_radius, m_radius);
        painter.drawPath(path.simplified());

        // 绘制小椭圆
        painter.setBrush(thumbColor);
        //painter.setOpacity(thumbOpacity);
        painter.drawEllipse(QPoint(qreal(width())/2 + 0.48,m_nY),width()/2,width()/2);
    }
}

// 鼠标按下事件
void QMaterialToggle::mousePressEvent(QMouseEvent *event)
{
    if (isEnabled()) {
        if (event->buttons() & Qt::LeftButton) {
            event->accept();
        } else {
            event->ignore();
        }
    }
}

// 鼠标释放事件 - 切换开关状态、发射toggled()信号
void QMaterialToggle::mouseReleaseEvent(QMouseEvent *event)
{
    if (isEnabled()) {
        if ((event->type() == QMouseEvent::MouseButtonRelease) && (event->button() == Qt::LeftButton)) {
            event->accept();
            m_bChecked = !m_bChecked;

            updateStatus();
            //切换开关状态、发射toggled()信号
            emit toggled(m_bChecked);
        } else {
            event->ignore();
        }
    }
    update();
}

// 大小改变事件
void QMaterialToggle::resizeEvent(QResizeEvent *event)
{
    if (m_orientation == Qt::Horizontal){
        if (m_bChecked) {
            m_nX = width()-height() / 2;
        } else {
            m_nX = height() / 2;
        }
    } else {
        if (m_bChecked) {
            m_nY = height()-width() / 2;
        } else {
            m_nY = width() / 2;
        }
    }

    //m_nY = m_nHeight / 2;
    QWidget::resizeEvent(event);
}

// 默认大小
QSize QMaterialToggle::sizeHint() const
{
    return minimumSizeHint();
}

// 最小大小
QSize QMaterialToggle::minimumSizeHint() const
{
    return QSize(2 * (m_nHeight + m_nMargin), m_nHeight + 2 * m_nMargin);
}

void QMaterialToggle::updateStatus()
{
    QPoint center;
    qreal radius;
    QColor color;
    qreal ocaption;
    if (m_orientation == Qt::Horizontal)
    {
        if (m_bChecked) {
            m_nX = width()-height() / 2;
        } else {
            m_nX = height() / 2;
        }
        center = QPoint( m_nX,height()/2);
        radius = height();
    } else {
        if (m_bChecked) {
            m_nY = height()-width() / 2;
        } else {
            m_nY = width() / 2;
        }
        center = QPoint( width()/2,m_nY);
        radius = width();
    }
    if(m_bChecked){
        color = QColor("#2c9dfc");
        color.setAlphaF(0.4);
        //ocaption = 0.4;
    } else {
        color = QColor("#011222");
        color.setAlphaF(0.1);
        //ocaption = 0.1;
    }
    //获取子窗口在父窗口的相对坐标
    QPoint point = this->mapTo(this->window(),this->geometry().topLeft());

    //添加动画
    MaterialRipple *ripple;
    ripple = new MaterialRipple(center,this,QEasingCurve::OutCirc);
    ripple->setRadiusEndValue(radius);
    ripple->setColor(color);
    //如果是在父窗口显示动画，则需要进行坐标转换
    ripple->movecenter(point);
    ripple->setOpacityStartValue(1);
    //动画显示窗口添加动画
    ndewidgetanimationmanager().getCirclemanager().findripplelay(this)->addRipple(ripple);

}

// 返回开关状态 - 打开：true 关闭：false
bool QMaterialToggle::isChecked() const
{
    return m_bChecked;
}

// 设置开关状态
void QMaterialToggle::setChecked(bool checked)
{
    m_bChecked = checked;
    updateStatus();
}

// 设置背景颜色
void QMaterialToggle::setBackgroundColor(QColor color)
{
    m_BackgroundColor = color;
}

void QMaterialToggle::setthumbColor(QColor color)
{
    m_thumbColor = color;
}

void QMaterialToggle::setCheckedColor(QColor color)
{
    m_checkedColor = color;
}

void QMaterialToggle::setOrientation(Qt::Orientation orientation)
{
    m_orientation = orientation;
}
}
